//=========================================================
// LPC1114 Project
//=========================================================
// File Name : main.c
// Function  : Main Routine
//---------------------------------------------------------
// Rev.01 2011.03.06 Munetomo Maruyama
//---------------------------------------------------------
// Copyright (C) 2010-2011 Munetomo Maruyama
//=========================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include "array_com.h"
#include "color_led.h"
#include "fixedpoint.h"
#include "oled.h"
#include "systick.h"
#include "utility.h"

#include <math.h>

//=====================
// Calculate PI
//=====================
//-------------------------------------------
// Select one Algorithm from followings
#define Spigot // USE ONLY ONE [OB/MB]
//#define BBP
//#define Simon_Plouffe_0
//#define Simon_Plouffe_1
//-------------------------------------------

#ifdef Spigot
//========================================
// Spigot Algorithm : USE ONLY ONE [OB/MB]
//========================================
//------------------------------------------------------------------------------------------
// This Program is based on following resource :
// Jeremy Gibbons, Unbounded Spigot Algorithms for the Digits of Pi
// http://www.comlab.ox.ac.uk/jeremy.gibbons/publications/spigot.pdf
//------------------------------------------------------------------------------------------
// // 15000 digits by 133 bytes Program
// a[52514],b,c=52514,d,e,f=1e4,g,h;
// main(){for(;b=c-=14;h=printf("%04d", e+d/f))
// for(e=d%=f;g=--b*2;d/=g)d=d*b+f*(h?a[b]:f/5),a[b]=d%--g;}
//------------------------------------------------------------------------------------------
#define DIGITS 256 // should be multiple of 4
#define ARRAYSIZE  (((DIGITS / 4) + 1) * 14)
//
int a[ARRAYSIZE];
int b;
int c = ARRAYSIZE;
int d = 0;
int e;
int f = 10000;
int g;
int h = 0;
//
int main(void)
{
    //
    // Initialization
    //
    Init_SysTick();
    Init_Color_LED();
    Init_OLED();
    //
    // Calculation
    //
    for(c = ARRAYSIZE - 14; c > 0; c = c - 14)
    {
        d %= f;
        e = d;
        for(b = c - 1; b > 0; --b)
        {
            g = 2 * b - 1;
            d = d * b + f * ((h)? a[b]:(f/5));
            a[b] = d % g;
            d /= g;
        }
        h = 4; // "first" flag
        OLED_printf("%04d", e + d / f);
    }
    return 0;
}
#endif

#ifdef BBP
//=======================================
//　BBP(Bailey-Borwein-Plouffe)　Algorithm
//=======================================
//------------------------------------------------------------------------------------------
// This Program is based on following resource : http://www.experimentalmath.info/bbp-codes/
//------------------------------------------------------------------------------------------
//  This program implements the BBP algorithm to generate a few hexadecimal
//  digits beginning immediately after a given position id, or in other words
//  beginning at position id + 1.  On most systems using IEEE 64-bit floating-
//  point arithmetic, this code works correctly so long as d is less than
//  approximately 1.18 x 10^7.  If 80-bit arithmetic can be employed, this limit
//  is significantly higher.  Whatever arithmetic is used, results for a given
//  position id can be checked by repeating with id-1 or id+1, and verifying
//  that the hex digits perfectly overlap with an offset of one, except possibly
//  for a few trailing digits.  The resulting fractions are typically accurate
//  to at least 11 decimal digits, and to at least 9 hex digits.
//
//  David H. Bailey     2006-09-08 */
//------------------------------------------------------------------------------------------
#define DIGITS_PER_PAGE 100
#define DIGITS_PER_LINE 10
//---------------------------------------------------------------------
// This returns, in chx, the first nhx hex digits of the fraction of x.
//---------------------------------------------------------------------
void ihex (double x, int nhx, char chx[])
{
  int i;
  double y;
  char hx[] = "0123456789ABCDEF";

  y = fabs (x);

  for (i = 0; i < nhx; i++){
    y = 16. * (y - floor (y));
    chx[i] = hx[(int) y];
  }
}

//-----------------------------------------------------------------
// expm = 16^p mod ak.  This routine uses the left-to-right binary
// exponentiation scheme.
//-----------------------------------------------------------------
double expm (double p, double ak)
{
  int i, j;
  double p1, pt, r;
#define ntp 25
  static double tp[ntp];
  static int tp1 = 0;

/*  If this is the first call to expm, fill the power of two table tp. */

  if (tp1 == 0) {
    tp1 = 1;
    tp[0] = 1.;

    for (i = 1; i < ntp; i++) tp[i] = 2. * tp[i-1];
  }

  if (ak == 1.) return 0.;

/*  Find the greatest power of two less than or equal to p. */

  for (i = 0; i < ntp; i++) if (tp[i] > p) break;

  pt = tp[i-1];
  p1 = p;
  r = 1.;

/*  Perform binary exponentiation algorithm modulo ak. */

  for (j = 1; j <= i; j++){
    if (p1 >= pt){
      r = 16. * r;
      r = r - (int) (r / ak) * ak;
      p1 = p1 - pt;
    }
    pt = 0.5 * pt;
    if (pt >= 1.){
      r = r * r;
      r = r - (int) (r / ak) * ak;
    }
  }

  return r;
}

//-----------------------------------------------------------
// This routine evaluates the series  sum_k 16^(id-k)/(8*k+m)
// using the modular exponentiation technique.
//-----------------------------------------------------------
double series (int m, int id)
{
  int k;
  double ak, p, s, t;
#define eps 1e-17

  /*  Sum the series up to id. */

  s = 0.0;
  for (k = 0; k < id; k++)
  {
    ak = 8 * k + m;
    p = id - k;
    t = expm (p, ak);
    s = s + t / ak;
    s = s - (int) s;
  }

/*  Compute a few terms where k >= id. */

  for (k = id; k <= id + 100; k++){
    ak = 8 * k + m;
    t = pow (16., (double) (id - k)) / ak;
    if (t < eps) break;
    s = s + t;
    s = s - (int) s;
  }
  return s;
}

//------------------------------------------
// Generate 10 digit PI value in hexadecimal
//------------------------------------------
void partial_pi(int n, uint8_t *buf)
{
    double pid, s1, s2, s3, s4;
    int id = n - 1;
    #define NHX 16
    //char chx[NHX];

  /*  id is the digit position.  Digits generated follow immediately after id. */

    s1 = series (1, id);
    s2 = series (4, id);
    s3 = series (5, id);
    s4 = series (6, id);
    pid = 4. * s1 - 2. * s2 - s3 - s4;
    pid = pid - (int) pid + 1.;
  //ihex (pid, NHX, chx);
  //printf (" position = %i\n fraction = %.15f \n hex digits =  %10.10s\n",
  //id, pid, chx);
    ihex (pid, NHX, (char*) buf);
}
//-----------------------
// Main Routine
//-----------------------
int main(void)
{
    uint32_t i;
    uint32_t page_start;
    uint32_t page_step;
    uint32_t partial_start;
    uint32_t digit;
    uint8_t  buf[32];
    //
    // Initialization
    //
    Init_SysTick();
    Init_Array_COM();
    Array_COM_ID_Assignment();
    //
    Init_Color_LED();
    Init_OLED();
    //
    // Show Processing Element Node
    //
    OLED_printf_Font(OLED_FONT_SMALL);
    OLED_printf_Position(0, 0);
    OLED_printf_Color(OLED_YEL, OLED_BLK);
    OLED_printf("<Calculate PI>\n");
    OLED_printf_Color(OLED_GRN, OLED_BLK);
    OLED_printf("Node(Total=%d)\n", Array_COM_Total_Nodes());
    OLED_printf("  ID=%d\n", Array_COM_Get_ID());
    OLED_printf("  (x,y)=(%d,%d)\n", Array_COM_Get_ID_PosX(), Array_COM_Get_ID_PosY());
    //
    // My Role is...
    //
    page_start = (Array_COM_Get_ID() * DIGITS_PER_PAGE + 1);
    page_step  = Array_COM_Total_Nodes() * DIGITS_PER_PAGE;
    partial_start = page_start;
    digit = page_start;
    //
    // Calculate PI
    //
    // forever
    while(1)
    {
        // new page
        OLED_printf_Color(OLED_MAG, OLED_BLK);
        OLED_printf_Position(0, 5);
        OLED_printf("Position=%d\n", digit);
        // erase
        for (i = 0; i < 10; i++)
        {
            OLED_printf("                ");
        }
        OLED_printf_Position(0, 6);
        OLED_printf_Color(OLED_WHT, OLED_BLK);
        // one page
        while(1)
        {
            partial_pi(partial_start, buf);
            //
            for (i = 0; i < DIGITS_PER_LINE; i++)
            {
                OLED_printf("%c", buf[i]);
                if (digit % DIGITS_PER_LINE == 0) OLED_printf("\n");
                digit++;
                if (digit >= (page_start + DIGITS_PER_PAGE)) break; // from internal loop
            }
            if (digit >= (page_start + DIGITS_PER_PAGE)) break; // from internal loop
            partial_start = partial_start + DIGITS_PER_LINE;
        }
        // next page
        page_start = page_start + page_step;
        partial_start = page_start;
        digit = page_start;
    }
    return 0;
}
#endif

#ifdef Simon_Plouffe_0
//=======================================
//　Simon Plouffe　Algorithm 0
//=======================================
//-------------------------------------------------------------------------------
// This Program is based on following resource.
// http://bellard.org/pi/pi.c
//-------------------------------------------------------------------------------
// Computation of the n'th decimal digit of \pi with very little memory.
// Written by Fabrice Bellard on January 8, 1997.
//
// We use a slightly modified version of the method described by Simon
// Plouffe in "On the Computation of the n'th decimal digit of various
// transcendental numbers" (November 1996). We have modified the algorithm
// to get a running time of O(n^2) instead of O(n^3log(n)^3).
//
// This program uses mostly integer arithmetic. It may be slow on some
// hardwares where integer multiplications and divisons must be done
// by software. We have supposed that 'int' has a size of 32 bits. If
// your compiler supports 'long long' integers of 64 bits, you may use
// the integer version of 'mul_mod' (see HAS_LONG_LONG).
//-------------------------------------------------------------------------------
#define DIGITS_PER_PAGE 90
#define DIGITS_PER_LINE 9
#define mul_mod(a,b,m) (( (long long) (a) * (long long) (b) ) % (m))
//#define mul_mod(a,b,m) fmod( (double) a * (double) b, m)
//--------------------------------
// Returns the inverse of x mod y
//--------------------------------
int inv_mod(int x, int y)
{
    int q, u, v, a, c, t;

    u = x;
    v = y;
    c = 1;
    a = 0;
    do
    {
        q = v / u;
        t = c;
        c = a - q * c;
        a = t;
        t = u;
        u = v - q * u;
        v = t;
    } while (u != 0);
    a = a % y;
    if (a < 0) a = y + a;
    return a;
}
//----------------------
// Returns (a^b) mod m
//----------------------
int pow_mod(int a, int b, int m)
{
    int r, aa;

    r = 1;
    aa = a;
    while (1)
    {
        if (b & 1) r = mul_mod(r, aa, m);
        b = b >> 1;
        if (b == 0) break;
        aa = mul_mod(aa, aa, m);
    }
    return r;
}
//----------------------------
// Returns true if n is prime
//----------------------------
int is_prime(int n)
{
    int r, i;
    if ((n % 2) == 0) return 0;
    r = (int)(sqrt(n));
    for(i = 3; i <= r; i += 2) if ((n % i) == 0) return 0;
    return 1;
}
//---------------------------------------------
// Returns the prime number immediately after n
//---------------------------------------------
int next_prime(int n)
{
    do
    {
        n++;
    }
    while (!is_prime(n));
    return n;
}
//---------------------------------------------
// Finds out nine digits of pi starting from n
//---------------------------------------------
void partial_pi(int n, uint8_t *buf)
{
    int av, a, vmax, N, num, den, k, kq, kq2, t, v, s, i;
    double sum;

    N=(int)((n + 20) * log(10) / log(2));
    sum=0;
    for(a = 3; a <= (2*N); a = next_prime(a))
    {
        vmax=(int)(log(2*N) / log(a));
        av = 1;
        for(i = 0; i < vmax; i++) av = av * a;
        s = 0;
        num = 1;
        den = 1;
        v = 0;
        kq = 1;
        kq2 = 1;
        for(k = 1; k <= N; k++)
        {
            t = k;
            if (kq >= a)
            {
                do
                {
                    t = t / a;
                    v--;
                }
                while ((t % a) == 0);
                kq = 0;
            }
            kq++;
            num = mul_mod(num, t, av);
            t = (2 * k - 1);
            if (kq2 >= a)
            {
                if (kq2 == a)
                {
                    do
                    {
                        t = t / a;
                        v++;
                    }
                    while ((t % a) == 0);
                }
                kq2 -= a;
            }
            den = mul_mod(den, t, av);
            kq2 += 2;
            if (v > 0)
            {
                t = inv_mod(den, av);
                t = mul_mod(t, num, av);
                t = mul_mod(t, k, av);
                for(i = v; i < vmax; i++) t = mul_mod(t, a, av);
                s += t;
                if (s >= av) s -= av;
            }
        }
        t = pow_mod(10, n-1, av);
        s = mul_mod(s,t,av);
        sum = fmod(sum + (double) s / (double) av, 1.0);
    }
    // return result
    xsnprintf(buf, 9, "%09d",(int)(sum * 1e9));
}
//-----------------------
// Main Routine
//-----------------------
int main(void)
{
    uint32_t i;
    uint32_t page_start;
    uint32_t page_step;
    uint32_t partial_start;
    uint32_t digit;
    uint8_t  buf[32];
    //
    // Initialization
    //
    Init_SysTick();
    Init_Array_COM();
    Array_COM_ID_Assignment();
    //
    Init_Color_LED();
    Init_OLED();
    //
    // Show Processing Element Node
    //
    OLED_printf_Font(OLED_FONT_SMALL);
    OLED_printf_Position(0, 0);
    OLED_printf_Color(OLED_YEL, OLED_BLK);
    OLED_printf("<Calculate PI>\n");
    OLED_printf_Color(OLED_GRN, OLED_BLK);
    OLED_printf("Node(Total=%d)\n", Array_COM_Total_Nodes());
    OLED_printf("  ID=%d\n", Array_COM_Get_ID());
    OLED_printf("  (x,y)=(%d,%d)\n", Array_COM_Get_ID_PosX(), Array_COM_Get_ID_PosY());
    //
    // My Role is...
    //
    page_start = (Array_COM_Get_ID() * DIGITS_PER_PAGE + 1);
    page_step  = Array_COM_Total_Nodes() * DIGITS_PER_PAGE;
    partial_start = page_start;
    digit = page_start;
    //
    // Calculate PI
    //
    // forever
    while(1)
    {
        // new page
        OLED_printf_Color(OLED_MAG, OLED_BLK);
        OLED_printf_Position(0, 5);
        OLED_printf("Position=%d\n", digit);
        // erase
        for (i = 0; i < 10; i++)
        {
            OLED_printf("                ");
        }
        OLED_printf_Position(0, 6);
        OLED_printf_Color(OLED_WHT, OLED_BLK);
        // one page
        while(1)
        {
            partial_pi(partial_start, buf);
            //
            for (i = 0; i < DIGITS_PER_LINE; i++)
            {
                OLED_printf("%c", buf[i]);
                if (digit % DIGITS_PER_LINE == 0) OLED_printf("\n");
                digit++;
                if (digit >= (page_start + DIGITS_PER_PAGE)) break; // from internal loop
            }
            if (digit >= (page_start + DIGITS_PER_PAGE)) break; // from internal loop
            partial_start = partial_start + DIGITS_PER_LINE;
        }
        // next page
        page_start = page_start + page_step;
        partial_start = page_start;
        digit = page_start;
    }
    return 0;
}
#endif

#ifdef Simon_Plouffe_1
//=======================================
//　Simon Plouffe　Algorithm 1
//=======================================
//-------------------------------------------------------------------------------
// This Program is based on following resource.
// http://bellard.org/pi/pi1.c
//-------------------------------------------------------------------------------
// Computation of the n'th decimal digit of pi with very little memory.
// Written by Fabrice Bellard on February 26, 1997.
//
// We use a slightly modified version of the method described by Simon
// Plouffe in "On the Computation of the n'th decimal digit of various
// transcendental numbers" (November 1996). We have modified the algorithm
// to get a running time of O(n^2) instead of O(n^3log(n)^3).
//
// This program uses a variation of the formula found by Gosper in 1974 :
//
// pi = sum( (25*n-3)/(binomial(3*n,n)*2^(n-1)), n=0..infinity);
//
// This program uses mostly integer arithmetic. It may be slow on some
// hardwares where integer multiplications and divisons must be done by
// software. We have supposed that 'int' has a size of at least 32 bits. If
// your compiler supports 'long long' integers of 64 bits, you may use the
// integer version of 'mul_mod' (see HAS_LONG_LONG).
//-------------------------------------------------------------------------------
#define DIGITS_PER_PAGE 90
#define DIGITS_PER_LINE 9
// uncomment the following line to use 'long long' integers
#define HAS_LONG_LONG
//
#ifdef HAS_LONG_LONG
#define mul_mod(a,b,m) (( (long long) (a) * (long long) (b) ) % (m))
#else
#define mul_mod(a,b,m) fmod( (double) a * (double) b, m)
#endif

//--------------------------------
// Returns the inverse of x mod y
//--------------------------------
int inv_mod(int x, int y)
{
    int q, u, v, a, c, t;

    u = x;
    v = y;
    c = 1;
    a = 0;
    do
    {
        q = v / u;

        t = c;
        c = a - q * c;
        a = t;

        t = u;
        u = v - q * u;
        v = t;
    }
    while (u != 0);
    a = a % y;
    if (a < 0) a = y + a;
    return a;
}
//-------------------------------------------
// return the inverse of u mod v, if v is odd
//-------------------------------------------
int inv_mod2(int u, int v)
{
    int u1, u3, v1, v3, t1, t3;

    u1 = 1;
    u3 = u;

    v1 = v;
    v3 = v;

    if ((u & 1) != 0)
    {
        t1 = 0;
        t3 = -v;
        goto Y4;
    }
    else
    {
        t1 = 1;
        t3 = u;
    }

    do
    {
        do
        {
            if ((t1 & 1) == 0)
            {
                t1 = t1 >> 1;
                t3 = t3 >> 1;
            }
            else
            {
                t1 = (t1 + v) >> 1;
                t3 = t3 >> 1;
            }
            Y4:;
        } while ((t3 & 1) == 0);

        if (t3 >= 0)
        {
            u1 = t1;
            u3 = t3;
        }
        else
        {
            v1 = v - t1;
            v3 = -t3;
        }
        t1 = u1 - v1;
        t3 = u3 - v3;
        if (t1 < 0)
        {
            t1 = t1 + v;
        }
    }
    while (t3 != 0);
    return u1;
}
//-------------------
// return (a^b) mod m
//-------------------
int pow_mod(int a, int b, int m)
{
    int r, aa;

    r = 1;
    aa = a;
    while (1)
    {
        if (b & 1) r = mul_mod(r, aa, m);
        b = b >> 1;
        if (b == 0) break;
        aa = mul_mod(aa, aa, m);
    }
    return r;
}
//--------------------------
// return true if n is prime
//--------------------------
int is_prime(int n)
{
    int r, i;
    if ((n % 2) == 0) return 0;

    r = (int)(sqrt(n));
    for(i = 3; i <= r; i += 2) if ((n % i) == 0) return 0;
    return 1;
}
//--------------------------------------------
// return the prime number immediately after n
//--------------------------------------------
int next_prime(int n)
{
    do
    {
        n++;
    }
    while (!is_prime(n));
    return n;
}

#define DIVN(t,a,v,vinc,kq,kqinc) \
{                                 \
    kq += kqinc;                  \
    if (kq >= a)                  \
    {                             \
        do                        \
        {                         \
            kq -= a;              \
        }                         \
        while (kq >= a);          \
        if (kq == 0)              \
        {                         \
            do                    \
            {                     \
                t = t / a;        \
                v += vinc;        \
            }                     \
            while ((t % a) == 0); \
        }                         \
    }                             \
}

//---------------------------------------------
// Finds out nine digits of pi starting from n
//---------------------------------------------
void partial_pi(int n, uint8_t *buf)
{
    int av, a, vmax, N, num, den;
    int k, kq1, kq2, kq3, kq4, t, v, s, i, t1;
    double sum;

    N = (int)((n + 20) * log(10) / log(13.5));
    sum = 0;

    for(a = 2; a <= (3 * N); a = next_prime(a))
    {
        vmax = (int)(log(3 * N) / log(a));
        if (a == 2)
        {
            vmax = vmax + (N - n);
            if (vmax <= 0) continue;
        }
        av = 1;
        for(i = 0; i < vmax; i++) av = av * a;

        s = 0;
        den = 1;
        kq1 = 0;
        kq2 = -1;
        kq3 = -3;
        kq4 = -2;
        if (a == 2)
        {
            num = 1;
            v = -n;
        }
        else
        {
            num = pow_mod(2, n, av);
            v = 0;
        }

        for(k = 1; k <= N; k++)
        {
            t = 2 * k;
            DIVN(t, a, v, -1, kq1, 2);
            num = mul_mod(num, t, av);

            t = 2 * k - 1;
            DIVN(t, a, v, -1, kq2, 2);
            num = mul_mod(num, t, av);

            t = 3 * (3 * k - 1);
            DIVN(t, a, v, 1, kq3, 9);
            den = mul_mod(den, t, av);

            t = (3 * k - 2);
            DIVN(t, a, v, 1, kq4, 3);
            if (a != 2) t = t * 2; else v++;
            den = mul_mod(den, t, av);

            if (v > 0)
            {
                if (a != 2)
                    t = inv_mod2(den, av);
                else
                    t = inv_mod(den, av);
                t = mul_mod(t, num, av);
                for(i = v; i < vmax; i++) t = mul_mod(t, a, av);
                t1 = (25 * k - 3);
                t = mul_mod(t, t1, av);
                s += t;
                if (s >= av) s -= av;
            }
        }
        t = pow_mod(5, n-1, av);
        s = mul_mod(s, t, av);
        sum = fmod(sum + (double) s / (double) av, 1.0);
    }
    // return result
    xsnprintf(buf, 9, "%09d",(int)(sum * 1e9));
}
//-----------------------
// Main Routine
//-----------------------
int main(void)
{
    uint32_t i;
    uint32_t page_start;
    uint32_t page_step;
    uint32_t partial_start;
    uint32_t digit;
    uint8_t  buf[32];
    //
    // Initialization
    //
    Init_SysTick();
    Init_Array_COM();
    Array_COM_ID_Assignment();
    //
    Init_Color_LED();
    Init_OLED();
    //
    // Show Processing Element Node
    //
    OLED_printf_Font(OLED_FONT_SMALL);
    OLED_printf_Position(0, 0);
    OLED_printf_Color(OLED_YEL, OLED_BLK);
    OLED_printf("<Calculate PI>\n");
    OLED_printf_Color(OLED_GRN, OLED_BLK);
    OLED_printf("Node(Total=%d)\n", Array_COM_Total_Nodes());
    OLED_printf("  ID=%d\n", Array_COM_Get_ID());
    OLED_printf("  (x,y)=(%d,%d)\n", Array_COM_Get_ID_PosX(), Array_COM_Get_ID_PosY());
    //
    // My Role is...
    //
    page_start = (Array_COM_Get_ID() * DIGITS_PER_PAGE + 1);
    page_step  = Array_COM_Total_Nodes() * DIGITS_PER_PAGE;
    partial_start = page_start;
    digit = page_start;
    //
    // Calculate PI
    //
    // forever
    while(1)
    {
        // new page
        OLED_printf_Color(OLED_MAG, OLED_BLK);
        OLED_printf_Position(0, 5);
        OLED_printf("Position=%d\n", digit);
        // erase
        for (i = 0; i < 10; i++)
        {
            OLED_printf("                ");
        }
        OLED_printf_Position(0, 6);
        OLED_printf_Color(OLED_WHT, OLED_BLK);
        // one page
        while(1)
        {
            partial_pi(partial_start, buf);
            //
            for (i = 0; i < DIGITS_PER_LINE; i++)
            {
                OLED_printf("%c", buf[i]);
                if (digit % DIGITS_PER_LINE == 0) OLED_printf("\n");
                digit++;
                if (digit >= (page_start + DIGITS_PER_PAGE)) break; // from internal loop
            }
            if (digit >= (page_start + DIGITS_PER_PAGE)) break; // from internal loop
            partial_start = partial_start + DIGITS_PER_LINE;
        }
        // next page
        page_start = page_start + page_step;
        partial_start = page_start;
        digit = page_start;
    }
    return 0;
}
#endif

//=========================================================
// End of Program
//=========================================================
